%% Synthetic Lloyd mirror model with noise
%A function for creating the synthetic propagation field with the Lloyd
%mirror model and add noise

function [ pf_cpa] = LMmodelNoise(Pparam,n)
    f = Pparam.freq;                    % Frequency values
    df = Pparam.df;                     % Frequency resolution
    t_bet = Pparam.t_bet;               % Time block spacing 
    t_len = Pparam.t_len;               % Length of time blocks
    t_num = Pparam.t_num;               % Number of time blocks
    v_s = Pparam.vs;                    % Ship velocity
    d_cpa = Pparam.dcpa;                % Distance at closest point of approach
    d_w = Pparam.d(1);                  % Reciever depth
    d_s = Pparam.d(2);                  % Source depth
    d_b = Pparam.d(3);                  % Seafloor depth
    c_w = Pparam.cp(1);                 % Sound speed in water
    c_s = Pparam.cp(2);                 % Sound speed in sediment
    dens_w = Pparam.rho(1);             % Density in water
    dens_s = Pparam.rho(2);             % Density in sediment
    ftype = Pparam.ftype;               % Type of model for processing synthetic data
    octavband = Pparam.octavband;       % Type of octaveband averaging
    shippath = Pparam.shippath;         % Type of ship path
    sbal = Pparam.sbal;                 % How many of the t_num time blocks is before or after the cpa when using shippath 
    tstart = Pparam.tstart;             % Decides if the ship path is created from CPA, value 0, or from a point 'x' timeblocks away, where positive values are in after CPA direction
    
    %wave number
    k = 2*pi.*f./c_w;

    %% Create ship path
    %time between each timeblock measured and with v_s can find distence
    %between each timeblock and can creating the time series of the signal in range 
   if (v_s == 0)                        % Standing still
        rb = zeros(t_num+1,1);
   else
        if (shippath == 's')            % Ship path symetric around CPA
            rb_min = -t_len*t_bet*round(t_num/2)*v_s + tstart*v_s;
            rb_max = t_len*t_bet*round(t_num/2)*v_s + tstart*v_s;
            steps = (rb_max-rb_min)/t_num;
            rb = rb_min:steps:rb_max;
        elseif (shippath == 'b')        % Ship path before CPA
            rb_min = -t_len*t_bet*round(t_num)*v_s + tstart*v_s;
            rb_max = tstart*v_s;
            steps = (rb_max-rb_min)/t_num;
            rb = rb_min:steps:rb_max;
        elseif (shippath == 'a')        % Ship path after CPA
            rb_min = tstart*v_s;
            rb_max = t_len*t_bet*round(t_num)*v_s + tstart*v_s;
            steps = (rb_max-rb_min)/t_num;
            rb = rb_min:steps:rb_max;
        elseif (shippath == 'l')        % Ship path abit before CPA and a little after
            rb_min = -t_len*t_bet*round(t_num-sbal)*v_s + tstart*v_s;
            rb_max = t_len*t_bet*round(sbal)*v_s + tstart*v_s;
            steps = (rb_max-rb_min)/t_num;
            rb = rb_min:steps:rb_max;
        elseif (shippath == 'k')        % Ship path abit after CPA and a little before
            rb_min = -t_len*t_bet*round(sbal)*v_s + tstart*v_s;
            rb_max = t_len*t_bet*round(t_num - sbal)*v_s + tstart*v_s;
            steps = (rb_max-rb_min)/t_num;
            rb = rb_min:steps:rb_max;
        end
   end
    
   %% Pressure calculation
    %Calculate noise from the adjusted noise
    Noise = AmbNoise(f)*Pparam.Nadj;
    Nvar = Pparam.Nvar;
    
    
    pf_cpa = zeros(length(f),length(rb));
    if((ftype == 'n') || (ftype == 'w'))
        %normal
        for l = 1:length(f)
            for kt = 1:length(rb)
                %creat ship path as distance from the reciever
                rt = sqrt((rb(kt))^2 + d_cpa^2); 

                %LM-model with n number of ray pairs
                pp1 = zeros(1,n);
                pn1 = zeros(1,n);
                Rp1 = zeros(2,n);
                Rn1 = zeros(2,n);
                Vp1 = zeros(2,n);
                Vn1 = zeros(2,n);
                for i = 1:n
                    %Range calcualtion
                    [Rn1(1,i),Rn1(2,i), Rp1(1,i),Rp1(2,i)] = Range(i-1,d_w, d_s,d_b, rt);
                    %reflection calcualtion
                    Vn1(1,i) = refcoff(rt, Rn1(1,i), c_w, c_s, dens_w, dens_s);
                    Vn1(2,i) = refcoff(rt, Rn1(2,i), c_w, c_s, dens_w, dens_s);
                    Vp1(1,i) = refcoff(rt, Rp1(1,i), c_w, c_s, dens_w, dens_s);
                    Vp1(2,i) = refcoff(rt, Rp1(2,i), c_w, c_s, dens_w, dens_s);
                    %pressure calculation
                    [pp1(i), pn1(i)] = UpDown(i-1,k(l),Rp1(:,i),Rn1(:,i),Vp1(:,i),Vn1(:,i));
                end
                %The Lloyd mirror signal

                pf_cpa(l,kt) = abs(sum(pp1) + sum(pn1)) + Noise(l) + (Noise(l)*Nvar).*randn;    
            end
        end
        
        %% Test octaveband averaging
    elseif(ftype == 'o') 
        if (octavband == 'l')
            %octaveband averaging and linearization
            fomax = max(f);
            order = floor(10*log10(fomax))+1;
            norder = 1:order;
            octfreq = zeros(3,length(norder));
            for i = 1:length(norder)
                octfreq(2,i) = 10^(norder(i)/10);
                octfreq(1,i) = octfreq(2,i)*2^(1/6);
                octfreq(3,i) = octfreq(2,i)*2^(-1/6);
            end

            %find the index of the octavfreq
            indof = zeros(1,2*length(f));
            for i = 1:length(f)
                for j = 1:length(norder)
                   if (octfreq(3,j) <= f(i) && f(i) <= octfreq(1,j))
                       indof(2*i-1) = j;
                       indof(2*i) = j+1;
                   end
                end
            end

            freqk = octfreq(:,indof);
            %create matrix
            for kt = 1:length(rb)
                for l = 1:length(f)
                    kmax = round(freqk(1,2*l-1)/df);
                    kmin = round(freqk(3,2*l-1)/df);

                    resp = zeros(length(kmin:df:kmax),1);
                    for kf = kmin:df:kmax
                        k = 2*pi.*kf./c_w;
                        %creat ship path as distance from the reciever
                        rt = sqrt((rb(kt))^2 + d_cpa^2); 

                        %LM-model with n number of ray pairs
                        pp1 = zeros(1,n);
                        pn1 = zeros(1,n);
                        Rp1 = zeros(2,n);
                        Rn1 = zeros(2,n);
                        Vp1 = zeros(2,n);
                        Vn1 = zeros(2,n);
                        for i = 1:n
                            %Range calcualtion
                            [Rn1(1,i),Rn1(2,i), Rp1(1,i),Rp1(2,i)] = Range(i-1,d_w, d_s,d_b, rt);
                            %reflection calcualtion
                            Vn1(1,i) = refcoff(rt, Rn1(1,i), c_w, c_s, dens_w, dens_s);
                            Vn1(2,i) = refcoff(rt, Rn1(2,i), c_w, c_s, dens_w, dens_s);
                            Vp1(1,i) = refcoff(rt, Rp1(1,i), c_w, c_s, dens_w, dens_s);
                            Vp1(2,i) = refcoff(rt, Rp1(2,i), c_w, c_s, dens_w, dens_s);
                            %pressure calculation
                            [pp1(i), pn1(i)] = UpDown(i-1,k,Rp1(:,i),Rn1(:,i),Vp1(:,i),Vn1(:,i));
                        end
                        %The Lloyd mirror signal
                        resp(kf) = sum(pp1) + sum(pn1) + Noise(l) + (Noise(l)*Nvar).*randn;   
                    end
                    %The Lloyd mirror signal
                    pfo1 = abs((1/(kmax-kmin+1))*sum(resp(kmin:kmax)));

                    kmax = round(freqk(1,2*l)/df);
                    kmin = round(freqk(3,2*l)/df);
                    resp = zeros(length(kmin:df:kmax),1);
                    for kf = kmin:df:kmax
                        k = 2*pi.*kf./c_w;
                        %creat ship path as distance from the reciever
                        rt = sqrt((rb(kt))^2 + d_cpa^2); 

                        %LM-model with n number of ray pairs
                        pp1 = zeros(1,n);
                        pn1 = zeros(1,n);
                        Rp1 = zeros(2,n);
                        Rn1 = zeros(2,n);
                        Vp1 = zeros(2,n);
                        Vn1 = zeros(2,n);
                        for i = 1:n
                            %Range calcualtion
                            [Rn1(1,i),Rn1(2,i), Rp1(1,i),Rp1(2,i)] = Range(i-1,d_w, d_s,d_b, rt);
                            %reflection calcualtion
                            Vn1(1,i) = refcoff(rt, Rn1(1,i), c_w, c_s, dens_w, dens_s);
                            Vn1(2,i) = refcoff(rt, Rn1(2,i), c_w, c_s, dens_w, dens_s);
                            Vp1(1,i) = refcoff(rt, Rp1(1,i), c_w, c_s, dens_w, dens_s);
                            Vp1(2,i) = refcoff(rt, Rp1(2,i), c_w, c_s, dens_w, dens_s);
                            %pressure calculation
                            [pp1(i), pn1(i)] = UpDown(i-1,k,Rp1(:,i),Rn1(:,i),Vp1(:,i),Vn1(:,i));
                        end
                        %The Lloyd mirror signal
                        resp(kf) = sum(pp1) + sum(pn1) + Noise(l) + (Noise(l)*Nvar).*randn;   
                    end
                    %The Lloyd mirror signal
                    pfo2 = abs((1/(kmax-kmin+1))*sum(resp(kmin:kmax)));

                    %linearise the octave averaged signal
                    xmin = round(freqk(3,2*l-1));
                    xmax = round(freqk(1,2*l-1));
                    ymin = pfo1;
                    ymax = pfo2;
                    octmffreq = xmin:xmax;
                    octmfsig = zeros(length(octmffreq),1);
                    for j = 1:length(octmffreq)
                        octmfsig(j) = ymin + ((ymax-ymin)/(xmax-xmin))*((xmin:xmax)-xmin);
                        if octmffreq(j) == f(l)
                            ind = j;
                        end
                    end

                    pf_cpa(l,kt) = octmfsig(ind);
                end  
            end
        elseif (octavband == 'n')
            %octaveband averaging
            fomax = max(f);
            order = floor(10*log10(fomax));
            norder = 1:order;
            octfreq = zeros(3,length(norder));
            for i = 1:length(norder)
                octfreq(2,i) = 10^(norder(i)/10);
                octfreq(1,i) = octfreq(2,i)*2^(1/6);
                octfreq(3,i) = octfreq(2,i)*2^(-1/6);
            end

            %find the index of the octavfreq
            indof = zeros(1,length(f));
            for i = 1:length(f)
                for j = 1:length(norder)
                   if (octfreq(3,j) <= f(i) && f(i) <= octfreq(1,j))
                       indof(i) = j;
                   end
                end
            end

            freqk = octfreq(:,indof);
            for l = 1:length(f)
                kmax = round(freqk(1,l)/df);
                kmin = round(freqk(3,l)/df);
                for kt = 1:length(rb)
                    resp = zeros(length(kmin:df:kmax),1);
                    for kf = kmin:df:kmax
                        k = 2*pi.*kf./c_w;
                        %creat ship path as distance from the reciever
                        rt = sqrt((rb(kt))^2 + d_cpa^2); 

                        %LM-model with n number of ray pairs
                        pp1 = zeros(1,n);
                        pn1 = zeros(1,n);
                        Rp1 = zeros(2,n);
                        Rn1 = zeros(2,n);
                        Vp1 = zeros(2,n);
                        Vn1 = zeros(2,n);
                        for i = 1:n
                            %Range calcualtion
                            [Rn1(1,i),Rn1(2,i), Rp1(1,i),Rp1(2,i)] = Range(i-1,d_w, d_s,d_b, rt);
                            %reflection calcualtion
                            Vn1(1,i) = refcoff(rt, Rn1(1,i), c_w, c_s, dens_w, dens_s);
                            Vn1(2,i) = refcoff(rt, Rn1(2,i), c_w, c_s, dens_w, dens_s);
                            Vp1(1,i) = refcoff(rt, Rp1(1,i), c_w, c_s, dens_w, dens_s);
                            Vp1(2,i) = refcoff(rt, Rp1(2,i), c_w, c_s, dens_w, dens_s);
                            %pressure calculation
                            [pp1(i), pn1(i)] = UpDown(i-1,k,Rp1(:,i),Rn1(:,i),Vp1(:,i),Vn1(:,i));
                        end
                        %The Lloyd mirror signal
                        resp(kf) = sum(pp1) + sum(pn1);   
                    end
                    %The Lloyd mirror signal
                    pf_cpa(l,kt) = abs((1/(kmax-kmin+1))*sum(resp(kmin:kmax))) + Noise(l) + (Noise(l)*Nvar).*randn;
                end
            end
        end
    end
    
end

